import django.dispatch

from exceptions import *

__all__ = ['city_items_pre_import']

city_items_pre_import = django.dispatch.Signal(providing_args=['items'])

def filter_non_cities(sender, items, **kwargs):
    if 'PPL' not in items[7]:
        raise InvalidItems()
city_items_pre_import.connect(filter_non_cities)



from ajax_select import LookupChannel
from django.utils.html import escape
from django.db.models import Q

from .models import *

class StandardLookupChannel(LookupChannel):
    def format_match(self,obj):
        """ (HTML) formatted item for displaying item in the dropdown """
        return self.get_result(obj)

    def format_item_display(self,obj):
        """ (HTML) formatted item for displaying item in the selected deck area """
        return get_result(obj)

class CountryLookup(StandardLookupChannel):
    model = Country

    def get_query(self, q, request):
        return Country.objects.filter(
            Q(name__icontains=q) |
            Q(name_ascii__icontains=q)
        ).distinct()

class CityLookup(StandardLookupChannel):
    model = City

    def get_query(self, q, request):
        return City.objects.filter(
            Q(name__icontains=q) |
            Q(ascii_name__icontains=q)
        ).select_related('country').distinct()
    
    def get_result(self, obj):
        """ The text result of autocompleting the entered query """
        return u'%s (%s)' % (obj.name, obj.country.name)



import urllib
import time
import os
import os.path
import logging
import zipfile
import optparse

from django.db import models
from django.core.management.base import BaseCommand
from django.utils.encoding import force_unicode

from ...exceptions import *
from ...signals import *
from ...models import *
from ...settings import *

class Command(BaseCommand):
    args = '''
[--force-all] [--force-import-all \\]
                              [--force-import allCountries.txt cities15000.txt ...] \\
                              [--force allCountries.txt cities15000.txt ...]
    '''.strip()
    help = '''
Download all files in CITIES_LIGHT_COUNTRY_SOURCES if they were updated or if 
--force-all option was used.
Import country data if they were downloaded or if --force-import-all was used.

Same goes for CITIES_LIGHT_CITY_SOURCES.

It is possible to force the download of some files which have not been updated
on the server:

    manage.py --force cities15000.txt countryInfo.txt

It is possible to force the import of files which weren't downloaded using the 
--force-import option:

    manage.py --force-import cities15000.txt countryInfo.txt
    '''.strip()

    logger = logging.getLogger('cities_light')

    option_list = BaseCommand.option_list + (
        optparse.make_option('--force-import-all', action='store_true', default=False,
            help='Import even if files are up-to-date.'
        ),
        optparse.make_option('--force-all', action='store_true', default=False,
            help='Download and import if files are up-to-date.'
        ),
        optparse.make_option('--force-import', action='append', default=[],
            help='Import even if filenames matching FORCE_IMPORT are up-to-date.'
        ),
        optparse.make_option('--force', action='append', default=[],
            help='Download and import if filenames matching FORCE are up-to-date.'
        ),
    )

    def handle(self, *args, **options):
        if not os.path.exists(DATA_DIR):
            self.logger.info('Creating %s' % DATA_DIR)
            os.mkdir(DATA_DIR)

        for url in SOURCES:
            destination_file_name = url.split('/')[-1]
            destination_file_path = os.path.join(DATA_DIR, destination_file_name)
            
            force = options['force_all'] or destination_file_name in options['force']
            downloaded = self.download(url, destination_file_path, force)
            
            if destination_file_name.split('.')[-1] == 'zip':
                destination_file_name = destination_file_name.replace('zip', 'txt')
                self.extract(destination_file_path, destination_file_name)
                destination_file_path = os.path.join(DATA_DIR, destination_file_name)

            force_import = options['force_import_all'] or \
                destination_file_name in options['force_import']
            if downloaded or force_import:
                self.logger.info('Importing %s' % destination_file_name)

                if url in CITY_SOURCES:
                    self.city_import(destination_file_path)
                elif url in COUNTRY_SOURCES:
                    self.country_import(destination_file_path)
  
    def download(self, url, path, force=False):
        remote_file = urllib.urlopen(url)
        remote_time = time.strptime(remote_file.headers['last-modified'], 
            '%a, %d %b %Y %H:%M:%S %Z')
        remote_size = int(remote_file.headers['content-length'])

        if os.path.exists(path) and not force:
            local_time = time.gmtime(os.path.getmtime(path))
            local_size = os.path.getsize(path)

            if local_time >= remote_time and local_size == remote_size:
                self.logger.warning('Assuming local download is up to date for %s' % url)
                return False

        self.logger.info('Downloading %s into %s' % (url, path))
        with open(path, 'wb') as local_file:
            chunk = remote_file.read()
            while chunk:
                local_file.write(chunk)
                chunk = remote_file.read()
        
        return True

    def extract(self, zip_path, file_name):
        destination = os.path.join(DATA_DIR, file_name)
        
        self.logger.info('Extracting %s from %s into %s' % (file_name, zip_path, destination))

        with zipfile.ZipFile(zip_path) as zip_file:
            with open(destination, 'wb') as destination_file:
                destination_file.write(zip_file.read(file_name))

    def parse(self, file_path):
        file = open(file_path, 'r')
        line = True

        while line:
            line = file.readline().strip()
            
            if len(line) < 1 or line[0] == '#':
                continue

            yield [e.strip() for e in line.split('\t')]

    def _get_country(self, code2):
        '''
        Simple lazy identity map for code2->country
        '''
        if not hasattr(self, '_country_codes'):
            self._country_codes = {}

        if code2 not in self._country_codes.keys():
            self._country_codes[code2] = Country.objects.get(code2=code2)
        
        return self._country_codes[code2]

    def country_import(self, file_path):
        for items in self.parse(file_path):
            try:
                country = Country.objects.get(code2=items[0])
            except Country.DoesNotExist:
                country = Country(code2=items[0])

            country.name = items[4]
            country.code3 = items[1]
            country.continent = items[8]
            country.tld = items[9][1:] # strip the leading dot
            country.save()

    def city_import(self, file_path):
        for items in self.parse(file_path):
            try:
                city_items_pre_import.send(sender=self, items=items)
            except InvalidItems:
                continue
            
            kwargs = dict(name=items[1], country=self._get_country(items[8]))

            try:
                city = City.objects.get(**kwargs)
            except City.DoesNotExist:
                city = City(**kwargs)

            if not city.geoname_id: # city may have been added manually
                city.geoname_id = items[0]
                city.save()



import autocomplete_light

from models import *

autocomplete_light.register(Country)

class CityChannel(autocomplete_light.ChannelBase):
    def query_filter(self, results):
        results = super(CityChannel, self).query_filter(results)

        country_name = self.request.GET.get('country__name', False)
        if country_name:
            results = results.filter(country__name=country_name)

        country_pk = self.request.GET.get('country__pk', False)
        if country_pk:
            results = results.filter(country__pk=country_pk)

        return results

autocomplete_light.register(City, CityChannel)



from django import forms

from .models import Country, City

class CountryForm(forms.ModelForm):
    class Meta:
        model = Country
        exclude = ('name_ascii', 'slug')

class CityForm(forms.ModelForm):
    class Meta:
        model = City
        exclude = ('name_ascii', 'slug')



import re
from math import radians, sin, cos, acos
from django.db import models
from django.contrib import admin
from django.contrib.gis.geos import Point
    
first_cap_re = re.compile('(.)([A-Z][a-z]+)')
all_cap_re = re.compile('([a-z0-9])([A-Z])')
def un_camel(name):
    """
    Convert CamelCase to camel_case
    """
    s1 = first_cap_re.sub(r'\1_\2', name)
    return all_cap_re.sub(r'\1_\2', s1).lower()

earth_radius_km = 6371.009

def geo_distance(a, b):
    """Distance between two geo points in km. (p.x = long, p.y = lat)"""
    a_y = radians(a.y)
    b_y = radians(b.y)
    delta_x = radians(a.x - b.x)
    cos_x = (   sin(a_y) * sin(b_y) +
                cos(a_y) * cos(b_y) * cos(delta_x))
    return acos(cos_x) * earth_radius_km
    
def create_model(name, fields=None, app_label='', module='', options=None, admin_opts=None):
    """
    Dynamically create model specified with args
    """
    class Meta:
        # Using type('Meta', ...) gives a dictproxy error during model creation
        pass

    if app_label:
        # app_label must be set using the Meta inner class
        setattr(Meta, 'app_label', app_label)

    # Update Meta with any options that were provided
    if options is not None:
        for key, value in options.iteritems():
            setattr(Meta, key, value)

    # Set up a dictionary to simulate declarations within a class
    attrs = {'__module__': module, 'Meta': Meta}

    # Add in any fields that were provided
    if fields:
        attrs.update(fields)

    # Create the class, which automatically triggers ModelBase processing
    model = type(name, (models.Model,), attrs)

    # Create an Admin class if admin options were provided
    if admin_opts is not None:
        class Admin(admin.ModelAdmin):
            pass
        for key, value in admin_opts:
            setattr(Admin, key, value)
        admin.site.register(model, Admin)

    return model
    


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Country'
        db.create_table('cities_light_country', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name_ascii', self.gf('django.db.models.fields.CharField')(max_length=200, db_index=True)),
            ('slug', self.gf('autoslug.fields.AutoSlugField')(unique_with=(), max_length=50, populate_from=None, db_index=True)),
            ('code2', self.gf('django.db.models.fields.CharField')(max_length=2, unique=True, null=True, blank=True)),
            ('code3', self.gf('django.db.models.fields.CharField')(max_length=3, unique=True, null=True, blank=True)),
            ('continent', self.gf('django.db.models.fields.CharField')(max_length=2, db_index=True)),
            ('tld', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=5, blank=True)),
            ('geoname_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal('cities_light', ['Country'])

        # Adding model 'City'
        db.create_table('cities_light_city', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200, db_index=True)),
            ('name_ascii', self.gf('django.db.models.fields.CharField')(max_length=200, db_index=True)),
            ('slug', self.gf('autoslug.fields.AutoSlugField')(unique_with=(), max_length=50, populate_from=None, db_index=True)),
            ('geoname_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('country', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cities_light.Country'])),
        ))
        db.send_create_signal('cities_light', ['City'])

        # Adding unique constraint on 'City', fields ['country', 'name']
        db.create_unique('cities_light_city', ['country_id', 'name'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'City', fields ['country', 'name']
        db.delete_unique('cities_light_city', ['country_id', 'name'])

        # Deleting model 'Country'
        db.delete_table('cities_light_country')

        # Deleting model 'City'
        db.delete_table('cities_light_city')


    models = {
        'cities_light.city': {
            'Meta': {'ordering': "['name']", 'unique_together': "(('country', 'name'),)", 'object_name': 'City'},
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cities_light.Country']"}),
            'geoname_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'db_index': 'True'}),
            'name_ascii': ('django.db.models.fields.CharField', [], {'max_length': '200', 'db_index': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': 'None', 'db_index': 'True'})
        },
        'cities_light.country': {
            'Meta': {'ordering': "['name']", 'object_name': 'Country'},
            'code2': ('django.db.models.fields.CharField', [], {'max_length': '2', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'code3': ('django.db.models.fields.CharField', [], {'max_length': '3', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'continent': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'}),
            'geoname_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'name_ascii': ('django.db.models.fields.CharField', [], {'max_length': '200', 'db_index': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': 'None', 'db_index': 'True'}),
            'tld': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '5', 'blank': 'True'})
        }
    }

    complete_apps = ['cities_light']



import unicodedata

from django.utils.encoding import force_unicode
from django.db.models import signals
from django.db import models
from django.template import defaultfilters
from django.utils.translation import ugettext as _

import autoslug

from settings import *

__all__ = ['Country','City', 'CONTINENT_CHOICES']

CONTINENT_CHOICES = (
    ('OC', _(u'Oceania')),
    ('EU', _(u'Europe')),
    ('AF', _(u'Africa')),
    ('NA', _(u'North America')),
    ('AN', _(u'Antarctica')),
    ('SA', _(u'South America')),
    ('AS', _(u'Asia')),
)

def set_name_ascii(sender, instance=None, **kwargs):
    if isinstance(instance.name, str):
        instance.name = force_unicode(instance.name)

    instance.name_ascii = unicodedata.normalize('NFKD', instance.name
        ).encode('ascii', 'ignore')

class Country(models.Model):
    name = models.CharField(max_length=200, unique=True)
    name_ascii = models.CharField(max_length=200, db_index=True)
    slug = autoslug.AutoSlugField(populate_from='name_ascii')

    code2 = models.CharField(max_length=2, null=True, blank=True, unique=True)
    code3 = models.CharField(max_length=3, null=True, blank=True, unique=True)
    continent = models.CharField(max_length=2, db_index=True, choices=CONTINENT_CHOICES)
    tld = models.CharField(max_length=5, blank=True, db_index=True)
    geoname_id = models.IntegerField(null=True, blank=True)
    
    class Meta:
        verbose_name_plural = _(u'countries')
        ordering = ['name']

    def __unicode__(self):
        return self.name
signals.pre_save.connect(set_name_ascii, sender=Country)

class City(models.Model):
    name = models.CharField(max_length=200, db_index=True)
    name_ascii = models.CharField(max_length=200, db_index=True)
    slug = autoslug.AutoSlugField(populate_from='name_ascii', 
        unique_with=('country__name',))
    
    geoname_id = models.IntegerField(null=True, blank=True)
    country = models.ForeignKey(Country)

    class Meta:
        unique_together = (('country', 'name'),)
        verbose_name_plural = _(u'cities')
        ordering = ['name']

        if not ENABLE_CITY:
            abstract = True
        
    def __unicode__(self):
        return self.name
signals.pre_save.connect(set_name_ascii, sender=City)



from django.contrib import admin

from .models import *
from .settings import *

class CountryAdmin(admin.ModelAdmin):
    list_display = (
        'name',
        'code2',
        'code3',
        'continent',
        'tld',
    )
    search_fields = (
        'name',
        'name_ascii',
        'code2',
        'code3',
        'tld'
    )
    list_filter = (
        'continent',
    )
admin.site.register(Country, CountryAdmin)

class CityAdmin(admin.ModelAdmin):
    list_display = (
        'name',
        'country',
    )
    search_fields = (
        'name',
        'name_ascii',
    )
    list_filter = (
        'country__continent',
        'country',
    )
if ENABLE_CITY:
    admin.site.register(City, CityAdmin)



from setuptools import setup, find_packages
import os

# Utility function to read the README file.
# Used for the long_description. It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django-cities-light',
    version='1.1',
    description='Simple alternative to django-cities',
    author='James Pic (original by Dan Carter (original by Ben Dowling))',
    author_email='jamespic@gmail.com@gmail.com',
    url='https://github.com/jpic/django-cities-light',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    long_description=read('README.rst'),
    license = 'MIT',
    keywords = 'django cities countries postal codes',
    install_requires=[
        'django_autoslug',
    ],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)




from django import forms

import autocomplete_light

from models import City

class CityAutocompleteWidget(forms.MultiWidget):
    def __init__(self, channel_name, attrs=None, **kwargs):
        widgets = (
            autocomplete_light.AutocompleteWidget('CountryChannel', max_items=1),
            autocomplete_light.AutocompleteWidget(channel_name, bootstrap='countrycity', 
                **kwargs),
        )
        super(CityAutocompleteWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            city = City.objects.get(pk=value)
            return [city.country.pk, value]
        return [None, None]
    
    def value_from_datadict(self, data, files, name):
        values = super(CityAutocompleteWidget, self).value_from_datadict(data, files, name)
        return values[1]



import os.path

from django.conf import settings

__all__ = ['COUNTRY_SOURCES', 'CITY_SOURCES', 'ENABLE_CITY', 'SOURCES', 'DATA_DIR']

COUNTRY_SOURCES = getattr(settings, 'CITIES_LIGHT_COUNTRY_SOURCES',
    ['http://download.geonames.org/export/dump/countryInfo.txt'])
CITY_SOURCES = getattr(settings, 'CITIES_LIGHT_CITY_SOURCES', 
    ['http://download.geonames.org/export/dump/cities15000.zip'])

ENABLE_CITY = getattr(settings, 'CITIES_LIGHT_ENABLE_CITY', True)

SOURCES = list(COUNTRY_SOURCES) + list(CITY_SOURCES)

DATA_DIR = getattr(settings, 'CITIES_LIGHT_DATA_DIR',
    os.path.normpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data')))

if hasattr(settings, 'AJAX_LOOKUP_CHANNELS'):
    settings.AJAX_LOOKUP_CHANNELS['cities_light_country'] = (
        'cities_light.lookups', 'CountryLookup')
    if ENABLE_CITY:
        settings.AJAX_LOOKUP_CHANNELS['cities_light_city'] = (
            'cities_light.lookups', 'CityLookup')
